local meld = require("__core__/lualib/meld")
local originalPipeToGroundEntity = data.raw["pipe-to-ground"]["pipe-to-ground"]
local originalPipeToGroundItem = data.raw["item"]["pipe-to-ground"]
local spaceAgeEnabled = mods["space-age"]

local function createPipeToGroundVariant(name, orderLetter, pipeConnections, recipeIngredients)
    local newPipeToGround = meld(table.deepcopy(originalPipeToGroundEntity), {
        name = name,
        minable = { result = name },
        fluid_box = {
            pipe_connections = meld.overwrite(pipeConnections)
        }
    })

    if spaceAgeEnabled then
        for i = 1, #newPipeToGround.fluid_box.pipe_connections do
            local pipeConnection = newPipeToGround.fluid_box.pipe_connections[i]

            if pipeConnection.connection_type and pipeConnection.connection_type == "underground" then
                pipeConnection.underground_collision_mask = { layers = { empty_space = true, lava_tile = true, water_tile = true } }
            end
        end
    end

    data:extend({
        newPipeToGround,
        meld(table.deepcopy(originalPipeToGroundItem), {
            name = name,
            order = "a[pipe]-" .. orderLetter .. "[" .. name .. "]",
            place_result = name
        }),
        {
            type = "recipe",
            name = name,
            enabled = false,
            ingredients = recipeIngredients,
            results = { { type = "item", name = name, amount = 1 } }
        }
    })

    table.insert(data.raw.technology["steam-power"].effects, {
        type = "unlock-recipe",
        recipe = name
    })
end

createPipeToGroundVariant(
    "pipe-to-ground-wn-su",
    "c",
    {
        { direction = defines.direction.west, position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 1 },
        { type = "item", name = "iron-plate",     amount = 2 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-ewn-su",
    "d",
    {
        { direction = defines.direction.east, position = { 0, 0 } },
        { direction = defines.direction.west, position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 1 },
        { type = "item", name = "iron-plate",     amount = 4 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-newn-su",
    "e",
    {
        { direction = defines.direction.north, position = { 0, 0 } },
        { direction = defines.direction.east,  position = { 0, 0 } },
        { direction = defines.direction.west,  position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 1 },
        { type = "item", name = "iron-plate",     amount = 6 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-wn-nsu",
    "f",
    {
        { direction = defines.direction.west, position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.north,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 2 },
        { type = "item", name = "iron-plate",     amount = 2 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-nn-swu",
    "g",
    {
        { direction = defines.direction.north, position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        },
        {
            connection_type = "underground",
            direction = defines.direction.west,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 2 },
        { type = "item", name = "iron-plate",     amount = 2 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-ewn-nsu",
    "h",
    {
        { direction = defines.direction.east, position = { 0, 0 } },
        { direction = defines.direction.west, position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.north,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 2 },
        { type = "item", name = "iron-plate",     amount = 4 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-nen-swu",
    "i",
    {
        { direction = defines.direction.north, position = { 0, 0 } },
        { direction = defines.direction.east,  position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        },
        {
            connection_type = "underground",
            direction = defines.direction.west,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 2 },
        { type = "item", name = "iron-plate",     amount = 4 }
    }
)

createPipeToGroundVariant(
    "pipe-to-ground-nn-eswu",
    "j",
    {
        { direction = defines.direction.north, position = { 0, 0 } },
        {
            connection_type = "underground",
            direction = defines.direction.east,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        },
        {
            connection_type = "underground",
            direction = defines.direction.south,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        },
        {
            connection_type = "underground",
            direction = defines.direction.west,
            position = { 0, 0 },
            max_underground_distance = 10,
            connection_category = { "default" }
        }
    },
    {
        { type = "item", name = "pipe-to-ground", amount = 3 },
        { type = "item", name = "iron-plate",     amount = 2 }
    }
)
